package geomorphometricswithgrids;

// java dependencies
import java.io.File;
import java.math.BigDecimal;
import java.util.HashSet;
// uk.ac.leeds.ccg.andyt.grids dependencies
import uk.ac.leeds.ccg.andyt.grids.exchange.ImageExporter;
import uk.ac.leeds.ccg.andyt.grids.exchange.ESRIAsciiGridExporter;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDouble;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDouble;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleFactory;
import uk.ac.leeds.ccg.andyt.grids.examples.Run;
import uk.ac.leeds.ccg.andyt.grids.process.Grid2DSquareCellProcessor;
import uk.ac.leeds.ccg.andyt.grids.process.Grid2DSquareCellProcessorDEM;
import uk.ac.leeds.ccg.andyt.grids.utilities.Utilities;
import uk.ac.leeds.ccg.andyt.grids.utilities.FileCreator;

public class ProcessorDEMRuns extends Run {
    
    long time;
    
    /** Creates a new ProcessorDEMRuns */
    public ProcessorDEMRuns() {
        super();
    }
    
    /**
     * Creates a new ProcessorDEMRuns using specified workspace.
     * WARNING: Files in the specified workspace may get overwritten.
     */
    public ProcessorDEMRuns( File workspace ) {
        super( workspace );
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws java.io.IOException {

		File workspace = new File( "M:/geog5060_prac7/geog5060_prac7/workspace/" );
		//File workspace = new File( "C:/Work/University/SoG/Teaching/Geog5060/GeomorphometricsWithGrids/workspace/" );
		ProcessorDEMRuns p = new ProcessorDEMRuns( workspace );
        p.run();
    }
    
    public void run() {
        time = System.currentTimeMillis();
        runTest();
        System.out.println( "Processing complete in " + Utilities.time( System.currentTimeMillis() - time ) );
    }
    
    public void runTest() {
        log( "runTest();" );
        // Set workspace to test
        grid2DSquareCellDoubleProcessor.setWorkspace( new File( grid2DSquareCellDoubleProcessor.getWorkspace(), "test" ), this.handleOutOfMemoryErrorFalse );
		log( "set workspace" );
        // Copy Data
        File workspace = grid2DSquareCellDoubleProcessor.getWorkspace();
		// Synthetic catchment1
        String filename = "catchment1.asc";
		String dataDirectory = "M:/geog5060_prac7/geog5060_prac7/data/";
		//String dataDirectory = "C:/Work/University/SoG/Teaching/Geog5060/GeomorphometricsWithGrids/data/";
		//log( "copy data" );
		File file = new File( dataDirectory, filename );
        //File copy = new File( workspace, filename );
        //FileCreator.copyFile( file, copy );
        //log( "Copied filed" );
		getMetrics1( file );
		//getMetrics1( copy );
		//getSlopeAndAspect( copy );
        //getFlowAccumulation( dataDirectory + "DEM/", dataDirectory + "FlowAccumulation/", filename );
        log( "Processing complete in " + Utilities.time( System.currentTimeMillis() - time ) );
    }
    
    public void getMetrics1( File file ) {
	    log( "getMetrics1( " + file.toString() + " )" );
        // Initialistaion
        boolean handleOutOfMemoryError = true;
        // Initialise dem as Grid2DSquareCellDoubleAbstract
        Grid2DSquareCellDoubleFactory grid2DSquareCellDoubleFactory = new Grid2DSquareCellDoubleFactory();
        Grid2DSquareCellDouble grid = ( Grid2DSquareCellDouble ) grid2DSquareCellDoubleFactory.createGrid2DSquareCellDouble( file );
        Grid2DSquareCellDoubleFactory gridFactory = grid2DSquareCellDoubleFactory;
        //Grid2DSquareCellDouble1Factory grid2DSquareCellDouble1Factory = this.grid2DSquareCellDoubleProcessor.getGrid2DSquareCellDouble1Factory();
        //Grid2DSquareCellDouble1 grid = ( Grid2DSquareCellDouble1 ) grid2DSquareCellDouble1Factory.createGrid2DSquareCellDouble( file );
        //Grid2DSquareCellDoubleAbstractFactory gridFactory = grid2DSquareCellDouble1Factory;
        String dataDirectory = this.grid2DSquareCellDoubleProcessor.getWorkspace().toString();
        grid.setName( file.getName().substring( 0, file.getName().length() - 4 ), handleOutOfMemoryError );
        log( grid.toString( handleOutOfMemoryError ) );
        
        // Write out an image for testing.
        //Grid2DSquareCellDoubleExporter.toAsciiFile( grid, new File( dataDirectory, grid.getName() + "_copy.asc" ) );
        System.out.println("Trying to export as png");
		ImageExporter imageExporter = new ImageExporter();
        imageExporter.toGreyScaleImage( grid, new File( dataDirectory, grid.getName() + ".png" ), "PNG" );
        
        // Initialise kernel parameters
        BigDecimal[] dimensions = grid.getDimensions();
        double cellsize = Double.valueOf( dimensions[ 0 ].toString() ).doubleValue();
        double weightIntersect = 1.0d;
        double weightFactor = 1.0d;
        int gridNcols = ( int ) grid.getNcols();
        int gridNrows = ( int ) grid.getNrows();
        log( "Initialised kernel parameters: cellize( " + cellsize + " ), weightIntersect( " + weightIntersect + " ), weightFactor( " + weightFactor + " ), gridNcols( " + gridNcols + " ), gridNrows( " + gridNrows + " )" );
        
        Grid2DSquareCellDouble dummyGrid = null;
        //int distances = 2;
        for ( int distances = 2; distances <= 8; distances *=2 ) {
            double distance = cellsize * ( double ) distances;
            Grid2DSquareCellProcessorDEM grid2DSquareCellProcessorDEM = new Grid2DSquareCellProcessorDEM();
            Grid2DSquareCellDouble[] metrics1 = grid2DSquareCellProcessorDEM.getMetrics1( grid, distance, weightIntersect, weightFactor, gridFactory );
            for ( int i = 0; i < metrics1.length; i ++ ) {
                dummyGrid = ( Grid2DSquareCellDouble ) metrics1[ i ];
                
                // Mask for edge effects
                // mask left
                grid2DSquareCellProcessorDEM.mask( dummyGrid, 0L, 0L, ( long ) ( gridNrows - 1 ), ( long ) ( distances - 1 ), handleOutOfMemoryError );
                // mask right
                grid2DSquareCellProcessorDEM.mask( dummyGrid, 0L, gridNcols - distances, gridNrows - 1, gridNcols - 1, handleOutOfMemoryError );
                // mask top
                grid2DSquareCellProcessorDEM.mask( dummyGrid, 0L, 0L, distances - 1, gridNcols - 1, handleOutOfMemoryError );
                // mask bottom
                grid2DSquareCellProcessorDEM.mask( dummyGrid, gridNrows - distances, 0, gridNrows - 1, gridNcols - 1, handleOutOfMemoryError );
                ESRIAsciiGridExporter eSRIAsciiGridExporter = new ESRIAsciiGridExporter();
                eSRIAsciiGridExporter.toAsciiFile( metrics1[ i ], new File( dataDirectory, metrics1[ i ].getName() + "_" + distances + ".asc" ), new BigDecimal(-9999.0) );
                imageExporter.toGreyScaleImage( metrics1[ i ], new File( dataDirectory, metrics1[ i ].getName() + "_" + distances + ".png" ), "PNG" );
            }
        }
    }
    
}
